import java.awt.event.*;
import javax.swing.*;
import javax.media.*;
import javax.media.format.*;
import javax.media.control.*;
import javax.media.protocol.*;
import java.awt.BorderLayout;

/**
 * @author Horst Eidenberger (hme@ims.tuwien.ac.at), adaptation: Heinrich
 *         Hussmann
 * @version 1
 */

/**
 * JFrame containing a video player that listens to ControllerEvents 
 * (processing chain related events).
 */
class VideoPlayerFrame extends JFrame implements ControllerListener {

	/** serialVersionUID is needed because this class is serializable.*/
	private static final long serialVersionUID = 1L;
	/** Processor (any type of processing chain) */
	Processor processor = null;
	/** Player (processing chain restricted to rendering streams)*/
	Player player = null;

	/**
	 * Creates a new VideoPlayerFrame and automatically loads the specified video file
	 * @param filename name of the video file
	 */
	public VideoPlayerFrame(String filename) {
		
		// basic window functionality
		setTitle("Gray Effect");
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent event) {
				processor.stop();
				processor.deallocate();
				System.exit(0);
			}
		});
		
		// create processor and control listener
		// catch exceptions in case something goes wrong
		try {
			// create a new processor and let the media locater located the input stream
			processor = Manager.createProcessor(new MediaLocator(filename));
			// tell the processor to send incoming events to this class (ControllerListener)
			processor.addControllerListener(this);
			
			processor.configure();
		} catch (Exception e) {
			System.out.println("Can't configure: " + e.getMessage());
		}
	}

	/**
	 * Incoming events are sent to this method
	 * @param event incoming event
	 */
	public synchronized void controllerUpdate(ControllerEvent event) {

		// handle incoming ConfigureCompleteEvents
		if (event instanceof ConfigureCompleteEvent) {
			TrackControl[] tracks = processor.getTrackControls();

			// found is false until the first video track is found
			boolean found = false;
			
			// run through all tracks
			for (int i = 0; i < tracks.length; i++) {
				
				// get the first video track
				if (!found && tracks[i].getFormat() instanceof VideoFormat) {
					
					// define processing chain
					Codec[] videoConversion = new Codec[] {
							new com.sun.media.codec.video.cinepak.JavaDecoder(),
							new GrayEffect() };

					try {
						// apply processing chain to video track
						tracks[i].setCodecChain(videoConversion);
						
						// define format of output stream
						processor.setContentDescriptor(new ContentDescriptor(
								ContentDescriptor.RAW));
						
						// do not look for further video tracks
						found = true;
						
					} catch (Exception e) {
						System.out.println("Can't set processing chain: "
								+ e.getMessage());
					}
				} 
			}

			if (found)
				// go to next state -> realized
				processor.realize();
			else {
				System.out.println("Attempt to realize processor failed.");
				System.exit(-1);
			}

		}

		// handle incoming RealizeCompleteEvents
		else if (event instanceof RealizeCompleteEvent) {
			try {
				// get output from processor and feed it into a new realized player
				player = Manager.createRealizedPlayer(processor.getDataOutput());
				
				// add player to the content pane and do the layout
				getContentPane().setLayout(new BorderLayout());
				getContentPane().add(player.getControlPanelComponent(),
						BorderLayout.SOUTH);
				getContentPane().add(player.getVisualComponent(),
						BorderLayout.NORTH);
				
				setSize(350, 300);
				setVisible(true);

				// start player and processor
				player.start();
				processor.start();
			} catch (Exception e) {
				System.out.println("Can't create player: " + e.getMessage());
			}
		}

		// handle incoming EndOfMediaEvents
		else if (event instanceof EndOfMediaEvent) {
			// stop player and processor
			player.stop();
			processor.stop();

			// restart player and processor
			processor.setMediaTime(new Time(0));
			player.setMediaTime(new Time(0));
			player.start();
			processor.start();
		}
	}

	public static void main(String[] argv) {
		String file = "http://www.medien.ifi.lmu.de/lehre/ss08/mt/uebungen/blatt4video.avi";
		new VideoPlayerFrame(file);
	}
}
